@@ -27,9 +27,11 @@ from darker.git import (
     RevisionRange,
     get_missing_at_revision,
     git_get_content_at_revision,
-    git_get_modified_files,
+    git_get_modified_python_files,
+    git_is_repository,
 )
 from darker.help import ISORT_INSTRUCTION
+from darker.highlighting import colorize
 from darker.import_sorting import apply_isort, isort
 from darker.linting import run_linters
 from darker.utils import GIT_DATEFORMAT, TextDocument, debug_dump, get_common_root
@@ -249,34 +251,39 @@ def print_diff(
             n=5,  # Black shows 5 lines of context, do the same
         )
     )
-
-    if sys.stdout.isatty():
-        try:
-            from pygments import highlight
-            from pygments.formatters import TerminalFormatter
-            from pygments.lexers import DiffLexer
-        except ImportError:
-            print(diff)
-        else:
-            print(highlight(diff, DiffLexer(), TerminalFormatter()))
-    else:
-        print(diff)
+    print(colorize(diff, "diff"))
 
 
 def print_source(new: TextDocument) -> None:
     """Print the reformatted Python source code"""
     if sys.stdout.isatty():
         try:
-            # pylint: disable=import-outside-toplevel
-            from pygments import highlight
-            from pygments.formatters import TerminalFormatter
-            from pygments.lexers.python import PythonLexer
+            (
+                highlight,
+                TerminalFormatter,  # pylint: disable=invalid-name
+                PythonLexer,
+            ) = _import_pygments()  # type: ignore
         except ImportError:
-            print(new.string)
+            print(new.string, end="")
         else:
-            print(highlight(new.string, PythonLexer(), TerminalFormatter()))
+            print(highlight(new.string, PythonLexer(), TerminalFormatter()), end="")
     else:
-        print(new.string)
+        print(new.string, end="")
+
+
+def _import_pygments():  # type: ignore
+    """Import within a function to ease mocking the import in unit-tests.
+
+    Cannot be typed as it imports parts of its own return type.
+    """
+    # pylint: disable=import-outside-toplevel
+    from pygments import highlight
+    from pygments.formatters import (  # pylint: disable=no-name-in-module
+        TerminalFormatter,
+    )
+    from pygments.lexers.python import PythonLexer
+
+    return highlight, TerminalFormatter, PythonLexer
 
 
 def main(argv: List[str] = None) -> int:
@@ -358,7 +365,7 @@ def main(argv: List[str] = None) -> int:
                 " Either --diff or --check must be used.",
             )
 
-    missing = get_missing_at_revision(paths, revrange.rev2)
+    missing = get_missing_at_revision(paths, revrange.rev2, root)
     if missing:
         missing_reprs = " ".join(repr(str(path)) for path in missing)
         rev2_repr = "the working tree" if revrange.rev2 == WORKTREE else revrange.rev2
@@ -379,10 +386,14 @@ def main(argv: List[str] = None) -> int:
         black_exclude = set()
     else:
         # In other modes, only process files which have been modified.
-        # These are relative to `root`:
-        changed_files_to_process = git_get_modified_files(
-            files_to_process, revrange, root
-        )
+        if git_is_repository(root):
+            changed_files_to_process = git_get_modified_python_files(
+                files_to_process, revrange, root
+            )
+        else:
+            changed_files_to_process = {
+                path.relative_to(root) for path in files_to_process
+            }
         black_exclude = {
             f for f in changed_files_to_process if root / f not in files_to_blacken
         }
