@@ -9,7 +9,7 @@ from datetime import datetime
 from functools import lru_cache
 from pathlib import Path
 from subprocess import DEVNULL, PIPE, CalledProcessError, check_output, run
-from typing import Iterable, List, Set, Tuple
+from typing import Dict, Iterable, List, Set, Tuple
 
 from darker.diff import diff_and_get_opcodes, opcodes_to_edit_linenums
 from darker.utils import GIT_DATEFORMAT, TextDocument
@@ -32,6 +32,21 @@ WORKTREE = ":WORKTREE:"
 PRE_COMMIT_FROM_TO_REFS = ":PRE-COMMIT:"
 
 
+def git_is_repository(path: Path) -> bool:
+    """Return ``True`` if ``path`` is inside a Git working tree"""
+    try:
+        lines = _git_check_output_lines(
+            ["rev-parse", "--is-inside-work-tree"], path, exit_on_error=False
+        )
+        return lines[:1] == ["true"]
+    except CalledProcessError as exc_info:
+        if exc_info.returncode != 128 or not exc_info.stderr.startswith(
+            "fatal: not a git repository"
+        ):
+            raise
+        return False
+
+
 def git_get_mtime_at_commit(path: Path, revision: str, cwd: Path) -> str:
     """Return the committer date of the given file at the given revision
 
@@ -62,7 +77,6 @@ def git_get_content_at_revision(path: Path, revision: str, cwd: Path) -> TextDoc
         abspath = cwd / path
         return TextDocument.from_file(abspath)
     cmd = ["show", f"{revision}:./{path.as_posix()}"]
-    logger.debug("[%s]$ %s", cwd, " ".join(cmd))
     try:
         return TextDocument.from_lines(
             _git_check_output_lines(cmd, cwd, exit_on_error=False),
@@ -158,18 +172,31 @@ def should_reformat_file(path: Path) -> bool:
     return path.exists() and path.suffix == ".py"
 
 
+@lru_cache(maxsize=1)
+def _make_git_env() -> Dict[str, str]:
+    """Create custom minimal environment variables to use when invoking Git
+
+    This makes sure that
+    - Git always runs in English
+    - ``$PATH`` is preserved (essential on NixOS)
+    - the environment is otherwise cleared
+
+    """
+    return {"LC_ALL": "C", "PATH": os.environ["PATH"]}
+
+
 def _git_check_output_lines(
     cmd: List[str], cwd: Path, exit_on_error: bool = True
 ) -> List[str]:
     """Log command line, run Git, split stdout to lines, exit with 123 on error"""
-    logger.debug("[%s]$ %s", cwd, " ".join(cmd))
+    logger.debug("[%s]$ git %s", cwd, " ".join(cmd))
     try:
         return check_output(
             ["git"] + cmd,
             cwd=str(cwd),
             encoding="utf-8",
             stderr=PIPE,
-            env={"LC_ALL": "C"},
+            env=_make_git_env(),
         ).splitlines()
     except CalledProcessError as exc_info:
         if not exit_on_error:
@@ -185,11 +212,12 @@ def _git_check_output_lines(
         sys.exit(123)
 
 
-def _git_exists_in_revision(path: Path, rev2: str) -> bool:
+def _git_exists_in_revision(path: Path, rev2: str, cwd: Path) -> bool:
     """Return ``True`` if the given path exists in the given Git revision
 
     :param path: The path of the file or directory to check
     :param rev2: The Git revision to look at
+    :param cwd: The Git repository root
     :return: ``True`` if the file or directory exists at the revision, or ``False`` if
              it doesn't.
 
@@ -197,11 +225,18 @@ def _git_exists_in_revision(path: Path, rev2: str) -> bool:
     # Surprise: On Windows, `git cat-file` doesn't work with backslash directory
     # separators in paths. We need to use Posix paths and forward slashes instead.
     cmd = ["git", "cat-file", "-e", f"{rev2}:{path.as_posix()}"]
-    result = run(cmd, check=False, stderr=DEVNULL, env={"LC_ALL": "C"})
+    logger.debug("[%s]$ %s", cwd, " ".join(cmd))
+    result = run(
+        cmd,
+        cwd=str(cwd),
+        check=False,
+        stderr=DEVNULL,
+        env=_make_git_env(),
+    )
     return result.returncode == 0
 
 
-def get_missing_at_revision(paths: Iterable[Path], rev2: str) -> Set[Path]:
+def get_missing_at_revision(paths: Iterable[Path], rev2: str, cwd: Path) -> Set[Path]:
     """Return paths missing in the given revision
 
     In case of ``WORKTREE``, just check if the files exist on the filesystem instead of
@@ -209,12 +244,13 @@ def get_missing_at_revision(paths: Iterable[Path], rev2: str) -> Set[Path]:
 
     :param paths: Paths to check
     :param rev2: The Git revision to look at, or ``WORKTREE`` for the working tree
+    :param cwd: The Git repository root
     :return: The set of file or directory paths which are missing in the revision
 
     """
     if rev2 == WORKTREE:
         return {path for path in paths if not path.exists()}
-    return {path for path in paths if not _git_exists_in_revision(path, rev2)}
+    return {path for path in paths if not _git_exists_in_revision(path, rev2, cwd)}
 
 
 def _git_diff_name_only(
@@ -267,10 +303,10 @@ def _git_ls_files_others(relative_paths: Set[Path], cwd: Path) -> Set[Path]:
     return {Path(line) for line in lines}
 
 
-def git_get_modified_files(
+def git_get_modified_python_files(
     paths: Iterable[Path], revrange: RevisionRange, cwd: Path
 ) -> Set[Path]:
-    """Ask Git for modified and untracked files
+    """Ask Git for modified and untracked ``*.py`` files
 
     - ``git diff --name-only --relative <rev> -- <path(s)>``
     - ``git ls-files --others --exclude-standard -- <path(s)>``
