@@ -23,25 +23,24 @@ import logging
 from contextlib import contextmanager
 from pathlib import Path
 from subprocess import PIPE, Popen
-from typing import IO, Generator, List, Optional, Set, Tuple, Union
+from typing import IO, Generator, List, Set, Tuple
 
 from darker.git import WORKTREE, EditedLinenumsDiffer, RevisionRange
+from darker.highlighting import colorize
 
 logger = logging.getLogger(__name__)
 
 
-def _parse_linter_line(
-    line: str, root: Path
-) -> Union[Tuple[Path, int], Tuple[None, None]]:
+def _parse_linter_line(line: str, root: Path) -> Tuple[Path, int, str, str]:
     # Parse an error/note line.
     # Given: line == "dir/file.py:123: error: Foo\n"
     # Sets: path = Path("abs/path/to/dir/file.py:123"
     #       linenum = 123
     #       description = "error: Foo"
     try:
-        location, _ = line[:-1].split(": ", 1)
-        path_str, linenum_bytes, *rest = location.split(":")
-        linenum = int(linenum_bytes)
+        location, description = line[:-1].split(": ", 1)
+        path_str, linenum_str, *rest = location.split(":")
+        linenum = int(linenum_str)
         if len(rest) > 1:
             raise ValueError("Too many colon-separated tokens")
         if len(rest) == 1:
@@ -53,10 +52,10 @@ def _parse_linter_line(
         # "Found XX errors in YY files (checked ZZ source files)"
         # "Success: no issues found in 1 source file"
         logger.debug("Unparseable linter output: %s", line[:-1])
-        return None, None
+        return Path(), 0, "", ""
     path_from_cwd = Path(path_str).absolute()
     path_in_repo = path_from_cwd.relative_to(root)
-    return path_in_repo, linenum
+    return path_in_repo, linenum, location + ":", description
 
 
 def _require_rev2_worktree(rev2: str) -> None:
@@ -98,27 +97,36 @@ def _check_linter_output(
 
 def run_linter(
     cmdline: str, root: Path, paths: Set[Path], revrange: RevisionRange
-) -> Optional[int]:
+) -> int:
     """Run the given linter and print linting errors falling on changed lines
 
     :param cmdline: The command line for running the linter
     :param root: The common root of all files to lint
     :param paths: Paths of files to check, relative to ``root``
     :param revrange: The Git revision rango to compare
-    :return: The number of modified lines with linting errors from this linter, or
-             ``None`` if there are no paths to check
+    :return: The number of modified lines with linting errors from this linter
 
     """
-    if not paths:
-        return None
     _require_rev2_worktree(revrange.rev2)
+    if not paths:
+        return 0
     error_count = 0
     edited_linenums_differ = EditedLinenumsDiffer(root, revrange)
     missing_files = set()
     with _check_linter_output(cmdline, root, paths) as linter_stdout:
+        prev_path, prev_linenum = None, 0
         for line in linter_stdout:
-            path_in_repo, linter_error_linenum = _parse_linter_line(line, root)
-            if path_in_repo is None or path_in_repo in missing_files:
+            (
+                path_in_repo,
+                linter_error_linenum,
+                location,
+                description,
+            ) = _parse_linter_line(line, root)
+            if (
+                path_in_repo is None
+                or path_in_repo in missing_files
+                or linter_error_linenum == 0
+            ):
                 continue
             try:
                 edited_linenums = edited_linenums_differ.compare_revisions(
@@ -129,7 +137,11 @@ def run_linter(
                 missing_files.add(path_in_repo)
                 continue
             if linter_error_linenum in edited_linenums:
-                print(line, end="")
+                if path_in_repo != prev_path or linter_error_linenum > prev_linenum + 1:
+                    print()
+                prev_path, prev_linenum = path_in_repo, linter_error_linenum
+                print(colorize(location, "lint_location"), end=" ")
+                print(colorize(description, "lint_description"))
                 error_count += 1
     return error_count
 
@@ -139,7 +151,7 @@ def run_linters(
     root: Path,
     paths: Set[Path],
     revrange: RevisionRange,
-) -> bool:
+) -> int:
     """Run the given linters on a set of files in the repository
 
     :param linter_cmdlines: The command lines for linter tools to run on the files
@@ -148,16 +160,15 @@ def run_linters(
                   files which have been modified in the repository between the given Git
                   revisions.
     :param revrange: The Git revisions to compare
-    :return: ``True`` if at least one linting error was found on a modified line
+    :return: Total number of linting errors found on modified lines
 
     """
-    some_linters_failed = False
-    for linter_cmdline in linter_cmdlines:
-        # 10. run linter subprocesses for all edited files (10.-13. optional)
-        # 11. diff the given revision and worktree (after isort and Black reformatting)
-        #     for each file reported by a linter
-        # 12. extract line numbers in each file reported by a linter for changed lines
-        # 13. print only linter error lines which fall on changed lines
-        if run_linter(linter_cmdline, root, paths, revrange):
-            some_linters_failed = True
-    return some_linters_failed
+    # 10. run linter subprocesses for all edited files (10.-13. optional)
+    # 11. diff the given revision and worktree (after isort and Black reformatting)
+    #     for each file reported by a linter
+    # 12. extract line numbers in each file reported by a linter for changed lines
+    # 13. print only linter error lines which fall on changed lines
+    return sum(
+        run_linter(linter_cmdline, root, paths, revrange)
+        for linter_cmdline in linter_cmdlines
+    )
