@@ -18,6 +18,25 @@ from darker.tests.helpers import raises_or_matches
 from darker.utils import GIT_DATEFORMAT, TextDocument
 
 
+def test_tmp_path_sanity(tmp_path):
+    """Make sure Pytest temporary directories aren't inside a Git repository"""
+    try:
+        result = git._git_check_output_lines(
+            ["rev-parse", "--absolute-git-dir"], tmp_path, exit_on_error=False
+        )
+    except CalledProcessError as exc_info:
+        if exc_info.returncode != 128 or not exc_info.stderr.startswith(
+            "fatal: not a git repository"
+        ):
+            raise
+    else:
+        output = "\n".join(result)
+        raise AssertionError(
+            f"Temporary directory {tmp_path} for tests is not clean."
+            f" There is a Git directory in {output}"
+        )
+
+
 @pytest.mark.parametrize(
     "revision_range, expect",
     [
@@ -176,7 +195,7 @@ def test_git_get_content_at_revision_obtain_file_content(
                 cwd=str(Path("/path")),
                 encoding="utf-8",
                 stderr=PIPE,
-                env={"LC_ALL": "C"},
+                env={"LC_ALL": "C", "PATH": os.environ["PATH"]},
             )
             for expected_call in expect_git_calls
         ]
@@ -377,13 +396,14 @@ def test_git_exists_in_revision_git_call(retval, expect):
     with patch.object(git, "run") as run:
         run.return_value.returncode = retval
 
-        result = git._git_exists_in_revision(Path("path.py"), "rev2")
+        result = git._git_exists_in_revision(Path("path.py"), "rev2", Path("."))
 
     run.assert_called_once_with(
         ["git", "cat-file", "-e", "rev2:path.py"],
+        cwd=".",
         check=False,
         stderr=DEVNULL,
-        env={"LC_ALL": "C"},
+        env={"LC_ALL": "C", "PATH": os.environ["PATH"]},
     )
     assert result == expect
 
@@ -410,7 +430,9 @@ def test_git_exists_in_revision(git_repo, rev2, path, expect):
     del_a = git_repo.get_hash()
     git_repo.add({"dir/b.py": None}, commit="Delete dir/b.py")
 
-    result = git._git_exists_in_revision(Path(path), rev2.format(add=add, del_a=del_a))
+    result = git._git_exists_in_revision(
+        Path(path), rev2.format(add=add, del_a=del_a), git_repo.root
+    )
 
     assert result == expect
 
@@ -431,6 +453,7 @@ def test_get_missing_at_revision(git_repo, rev2, expect):
     result = git.get_missing_at_revision(
         {Path("dir"), Path("dir/a.py"), Path("dir/b.py")},
         rev2.format(add=add, del_a=del_a),
+        git_repo.root,
     )
 
     assert result == expect
@@ -443,7 +466,7 @@ def test_get_missing_at_revision_worktree(git_repo):
     paths["dir/b.py"].unlink()
 
     result = git.get_missing_at_revision(
-        {Path("dir"), Path("dir/a.py"), Path("dir/b.py")}, git.WORKTREE
+        {Path("dir"), Path("dir/a.py"), Path("dir/b.py")}, git.WORKTREE, git_repo.root
     )
 
     assert result == {Path("dir/a.py"), Path("dir/b.py")}
@@ -508,8 +531,8 @@ def test_git_ls_files_others(git_repo):
     modify_paths={},
     paths=[],
 )
-def test_git_get_modified_files(git_repo, modify_paths, paths, expect):
-    """Tests for `darker.git.git_get_modified_files()`"""
+def test_git_get_modified_python_files(git_repo, modify_paths, paths, expect):
+    """Tests for `darker.git.git_get_modified_python_files()`"""
     root = Path(git_repo.root)
     git_repo.add(
         {
@@ -530,7 +553,9 @@ def test_git_get_modified_files(git_repo, modify_paths, paths, expect):
             absolute_path.write_bytes(content.encode("ascii"))
     revrange = git.RevisionRange("HEAD", ":WORKTREE:")
 
-    result = git.git_get_modified_files({root / p for p in paths}, revrange, cwd=root)
+    result = git.git_get_modified_python_files(
+        {root / p for p in paths}, revrange, cwd=root
+    )
 
     assert result == {Path(p) for p in expect}
 
@@ -677,11 +702,11 @@ def branched_repo(tmp_path_factory):
         expect={"mod_both.py", "mod_same.py", "mod_branch.py"},
     ),
 )
-def test_git_get_modified_files_revision_range(
+def test_git_get_modified_python_files_revision_range(
     _description, branched_repo, revrange, expect
 ):
-    """Test for :func:`darker.git.git_get_modified_files` with a revision range"""
-    result = git.git_get_modified_files(
+    """Test for :func:`darker.git.git_get_modified_python_files` with revision range"""
+    result = git.git_get_modified_python_files(
         [Path(branched_repo.root)],
         git.RevisionRange.parse_with_common_ancestor(revrange, branched_repo.root),
         Path(branched_repo.root),
@@ -765,12 +790,14 @@ def test_local_gitconfig_ignored_by_gitrepofixture(tmp_path):
     """Tests that ~/.gitconfig is ignored when running darker's git tests"""
     (tmp_path / "HEAD").write_text("ref: refs/heads/main")
 
-    with patch.dict(os.environ, {"HOME": str(tmp_path)}):
-        # Note: once we decide to drop support for git < 2.28, the HEAD file
-        # creation above can be removed, and setup can simplify to
-        # check_call("git config --global init.defaultBranch main".split())
-        check_call("git config --global init.templateDir".split() + [str(tmp_path)])
-        root = tmp_path / "repo"
-        root.mkdir()
-        git_repo = GitRepoFixture.create_repository(root)
-        assert git_repo.get_branch() == "master"
+    # Note: once we decide to drop support for git < 2.28, the HEAD file
+    # creation above can be removed, and setup can simplify to
+    # check_call("git config --global init.defaultBranch main".split())
+    check_call(
+        "git config --global init.templateDir".split() + [str(tmp_path)],
+        env={"HOME": str(tmp_path), "PATH": os.environ["PATH"]},
+    )
+    root = tmp_path / "repo"
+    root.mkdir()
+    git_repo = GitRepoFixture.create_repository(root)
+    assert git_repo.get_branch() == "master"
