@@ -8,13 +8,13 @@ from argparse import ArgumentError
 from pathlib import Path
 from textwrap import dedent
 from types import SimpleNamespace
-from unittest.mock import patch
+from unittest.mock import Mock, call, patch
 
 import pytest
-from black import find_project_root
 
 import darker.__main__
 import darker.import_sorting
+import darker.linting
 from darker.exceptions import MissingPackageError
 from darker.git import WORKTREE, RevisionRange
 from darker.tests.helpers import isort_present
@@ -37,9 +37,13 @@ def test_isort_option_without_isort(git_repo, caplog):
 
 
 @pytest.fixture
-def run_isort(git_repo, monkeypatch, caplog, request):
-    find_project_root.cache_clear()
+def run_isort(git_repo, monkeypatch, caplog, request, find_project_root_cache_clear):
+    """Fixture for running Darker with requested arguments and a patched `isort`
 
+    Provides an `run_isort.isort_code` mock object which allows checking whether and how
+    the `isort.code()` function was called.
+
+    """
     monkeypatch.chdir(git_repo.root)
     paths = git_repo.add({"test1.py": "original"}, commit="Initial commit")
     paths["test1.py"].write_bytes(b"changed")
@@ -434,14 +438,14 @@ def test_get_rev1_path(path, expect):
     dict(
         arguments=["--check", "--lint", "echo subdir/a.py:1: message"],
         # Windows compatible path assertion using `pathlib.Path()`
-        expect_stdout=[f"subdir/a.py:1: message {Path('/subdir/a.py')}"],
+        expect_stdout=["", f"subdir/a.py:1: message {Path('/subdir/a.py')}"],
         expect_retval=1,
     ),
     dict(
         arguments=["--diff", "--lint", "echo subdir/a.py:1: message"],
         # Windows compatible path assertion using `pathlib.Path()`
         expect_stdout=A_PY_DIFF_BLACK
-        + [f"subdir/a.py:1: message {Path('/subdir/a.py')}"],
+        + ["", f"subdir/a.py:1: message {Path('/subdir/a.py')}"],
         expect_retval=1,
     ),
     dict(
@@ -562,6 +566,57 @@ def test_main(
     assert retval == expect_retval
 
 
+def test_main_in_plain_directory(tmp_path, capsys):
+    """Darker works also in a plain directory tree"""
+    subdir_a = tmp_path / "subdir_a"
+    subdir_c = tmp_path / "subdir_b/subdir_c"
+    subdir_a.mkdir()
+    subdir_c.mkdir(parents=True)
+    (subdir_a / "non-python file.txt").write_text("not  reformatted\n")
+    (subdir_a / "python file.py").write_text("import  sys, os\nprint('ok')")
+    (subdir_c / "another python file.py").write_text("a  =5")
+    with patch.object(darker.linting, "run_linter") as run_linter:
+
+        retval = darker.__main__.main(
+            ["--diff", "--check", "--isort", "--lint", "my-linter", str(tmp_path)]
+        )
+
+    assert retval == 1
+    assert run_linter.call_args_list == [
+        call(
+            "my-linter",
+            tmp_path,
+            {
+                Path("subdir_a/python file.py"),
+                Path("subdir_b/subdir_c/another python file.py"),
+            },
+            RevisionRange(rev1="HEAD", rev2=":WORKTREE:"),
+        )
+    ]
+    output = capsys.readouterr().out
+    output = re.sub(
+        r"\d{4}-\d\d-\d\d \d\d:\d\d:\d\d\.\d\d\d\d\d\d", "<timestamp>", output
+    )
+    assert output == dedent(
+        """\
+        --- subdir_a/python file.py	<timestamp> +0000
+        +++ subdir_a/python file.py	<timestamp> +0000
+        @@ -1,2 +1,4 @@
+        -import  sys, os
+        -print('ok')
+        +import os
+        +import sys
+        +
+        +print("ok")
+        --- subdir_b/subdir_c/another python file.py	<timestamp> +0000
+        +++ subdir_b/subdir_c/another python file.py	<timestamp> +0000
+        @@ -1 +1 @@
+        -a  =5
+        +a = 5
+        """
+    )
+
+
 @pytest.mark.parametrize(
     "encoding, text", [(b"utf-8", b"touch\xc3\xa9"), (b"iso-8859-1", b"touch\xe9")]
 )
@@ -723,6 +778,50 @@ def test_modify_file(tmp_path, new_content, expect):
     assert result == expect
 
 
+@pytest.mark.kwparametrize(
+    dict(
+        new_content=TextDocument(lines=['print("foo")']),
+        tty=False,
+        with_pygments=False,
+        expect=('print("foo")\n',),
+    ),
+    dict(
+        new_content=TextDocument(lines=['print("foo")']),
+        tty=False,
+        with_pygments=True,
+        expect=('print("foo")\n',),
+    ),
+    dict(
+        new_content=TextDocument(lines=['print("foo")']),
+        tty=True,
+        with_pygments=False,
+        expect=('print("foo")\n',),
+    ),
+    dict(
+        new_content=TextDocument(lines=['print("foo")']),
+        tty=True,
+        with_pygments=True,
+        expect=(
+            '\x1b[36mprint\x1b[39;49;00m(\x1b[33m"\x1b[39;49;00mfoo'
+            + '\x1b[33m"\x1b[39;49;00m)\n',
+            '\x1b[36mprint\x1b[39;49;00m(\x1b[33m"\x1b[39;49;00m\x1b[33mfoo'
+            + '\x1b[39;49;00m\x1b[33m"\x1b[39;49;00m)\n',
+        ),
+    ),
+)
+def test_print_source(new_content, tty, with_pygments, expect, capsys):
+    """Highlight is applied only if tty, final newline is handled correctly."""
+    with patch("sys.stdout.isatty", Mock(return_value=tty)), patch(
+        "darker.__main__._import_pygments",
+        Mock(
+            return_value=darker.__main__._import_pygments(),
+            side_effect=None if with_pygments else ImportError(),
+        ),
+    ):
+        darker.__main__.print_source(new_content)
+        assert capsys.readouterr().out in expect
+
+
 def test_stdout_path_resolution(git_repo, capsys):
     """When using ``--stdout``, file paths are resolved correctly"""
     git_repo.add({"src/menu.py": "print ( 'foo' )\n"})
@@ -730,4 +829,4 @@ def test_stdout_path_resolution(git_repo, capsys):
     result = darker.__main__.main(["--stdout", "./src/menu.py"])
 
     assert result == 0
-    assert capsys.readouterr().out == 'print("foo")\n\n'
+    assert capsys.readouterr().out == 'print("foo")\n'
